@@ -19,8 +19,8 @@ ncvreg <- function(X, y, family=c("gaussian","binomial"), penalty=c("MCP", "SCAD
   scale <- attr(XX, "scale")
   nz <- which(scale > 1e-6)
   XX <- XX[ ,nz, drop=FALSE]
-  p <- ncol(XX)
   yy <- if (family=="gaussian") y - mean(y) else y
+  p <- ncol(XX)
   n <- length(yy)
   penalty.factor <- penalty.factor[nz]
   if (missing(lambda)) {
@@ -30,18 +30,21 @@ ncvreg <- function(X, y, family=c("gaussian","binomial"), penalty=c("MCP", "SCAD
     nlambda <- length(lambda)
     user.lambda <- TRUE
   }
-  
+
   ## Fit
   if (family=="gaussian") {
-    fit <- .C("cdfit_gaussian", double(p*nlambda), double(nlambda), integer(nlambda), as.double(XX), as.double(yy), as.integer(n), as.integer(p), penalty, as.double(lambda), as.integer(nlambda), as.double(eps), as.integer(max.iter), as.double(gamma), as.double(penalty.factor), as.double(alpha), as.integer(dfmax), as.integer(user.lambda | any(penalty.factor==0)))
-    b <- rbind(mean(y), matrix(fit[[1]],nrow=p))
-    loss <- fit[[2]]
-    iter <- fit[[3]]
+    b <- matrix(0, p, nlambda)
+    loss <- numeric(nlambda)
+    iter <- integer(nlambda)
+    .Call("cdfit_gaussian", b, loss, iter, XX, yy, penalty, lambda, eps, as.integer(max.iter), as.double(gamma), penalty.factor, alpha, as.integer(dfmax), as.integer(user.lambda | any(penalty.factor==0)))
+    b <- rbind(mean(y), b)
   } else if (family=="binomial") {
-    fit <- .C("cdfit_binomial",double(nlambda),double(p*nlambda),double(nlambda),integer(nlambda), as.double(XX), as.double(yy), as.integer(n), as.integer(p), penalty, as.double(lambda), as.integer(nlambda), as.double(eps), as.integer(max.iter), as.double(gamma), as.double(penalty.factor), as.double(alpha), as.integer(dfmax), as.integer(user.lambda | any(penalty.factor==0)), as.integer(warn))
-    b <- rbind(fit[[1]],matrix(fit[[2]],nrow=p))
-    loss <- fit[[3]]
-    iter <- fit[[4]]
+    b0 <- numeric(nlambda)
+    b <- matrix(0, p, nlambda)
+    loss <- numeric(nlambda)
+    iter <- integer(nlambda)
+    .Call("cdfit_binomial", b0, b, loss, iter, XX, as.numeric(yy), penalty, lambda, eps, as.integer(max.iter), as.double(gamma), penalty.factor, alpha, as.integer(dfmax), as.integer(user.lambda | any(penalty.factor==0)), as.integer(warn))
+    b <- rbind(b0, b)
   }
   
   ## Eliminate saturated lambda values, if any
@@ -51,7 +54,8 @@ ncvreg <- function(X, y, family=c("gaussian","binomial"), penalty=c("MCP", "SCAD
   lambda <- lambda[ind]
   loss <- loss[ind]
   if (warn & any(iter==max.iter)) warning("Algorithm failed to converge for all values of lambda")
-  
+
+  ## Local convexity?
   convex.min <- if (convex) convexMin(b, XX, penalty, gamma, lambda*(1-alpha), family) else NULL
   
   ## Unstandardize
